# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.request_options import RequestOptions
from ....types.conversation_tag_response_model import ConversationTagResponseModel
from ....types.get_conversation_tags_page_response_model import GetConversationTagsPageResponseModel
from .raw_client import AsyncRawTagsClient, RawTagsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TagsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTagsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTagsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTagsClient
        """
        return self._raw_client

    def assign(
        self,
        conversation_id: str,
        *,
        tag_ids: typing.Sequence[str],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Assign one or more conversation tags to a conversation. Tags that are already assigned are ignored. Tags must belong to the same workspace.

        Parameters
        ----------
        conversation_id : str

        tag_ids : typing.Sequence[str]
            Tag IDs to add to the conversation. Re-assigning an existing tag is a no-op.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.conversations.tags.assign(
            conversation_id="conversation_id",
            tag_ids=["tag_ids"],
        )
        """
        _response = self._raw_client.assign(conversation_id, tag_ids=tag_ids, request_options=request_options)
        return _response.data

    def unassign(
        self, conversation_id: str, tag_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Remove a single conversation tag from a conversation.

        Parameters
        ----------
        conversation_id : str

        tag_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.conversations.tags.unassign(
            conversation_id="conversation_id",
            tag_id="tag_id",
        )
        """
        _response = self._raw_client.unassign(conversation_id, tag_id, request_options=request_options)
        return _response.data

    def list(
        self,
        *,
        page_size: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetConversationTagsPageResponseModel:
        """
        List conversation tags for the workspace, ordered by most recently created first.

        Parameters
        ----------
        page_size : typing.Optional[int]
            How many conversation tags to return. Can not exceed 100.

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetConversationTagsPageResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.conversations.tags.list(
            page_size=1,
            cursor="cursor",
        )
        """
        _response = self._raw_client.list(page_size=page_size, cursor=cursor, request_options=request_options)
        return _response.data

    def create(
        self,
        *,
        title: str,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationTagResponseModel:
        """
        Create a new conversation tag for the workspace.

        Parameters
        ----------
        title : str
            Display title of the tag.

        description : typing.Optional[str]
            Optional free-text description.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationTagResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.conversations.tags.create(
            title="title",
        )
        """
        _response = self._raw_client.create(title=title, description=description, request_options=request_options)
        return _response.data

    def get(
        self, tag_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationTagResponseModel:
        """
        Get a conversation tag by ID.

        Parameters
        ----------
        tag_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationTagResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.conversations.tags.get(
            tag_id="tag_id",
        )
        """
        _response = self._raw_client.get(tag_id, request_options=request_options)
        return _response.data

    def delete(self, tag_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a conversation tag. Restricted to the tag owner or a workspace admin.

        Parameters
        ----------
        tag_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.conversations.tags.delete(
            tag_id="tag_id",
        )
        """
        _response = self._raw_client.delete(tag_id, request_options=request_options)
        return _response.data

    def update(
        self,
        tag_id: str,
        *,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationTagResponseModel:
        """
        Update a conversation tag's title and/or description. Restricted to the tag owner or a workspace admin.

        Parameters
        ----------
        tag_id : str

        title : typing.Optional[str]
            If provided, replaces the tag title. Omit to leave unchanged.

        description : typing.Optional[str]
            If provided, replaces the tag description. Omit to leave unchanged.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationTagResponseModel
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.conversations.tags.update(
            tag_id="tag_id",
        )
        """
        _response = self._raw_client.update(
            tag_id, title=title, description=description, request_options=request_options
        )
        return _response.data


class AsyncTagsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTagsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTagsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTagsClient
        """
        return self._raw_client

    async def assign(
        self,
        conversation_id: str,
        *,
        tag_ids: typing.Sequence[str],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Assign one or more conversation tags to a conversation. Tags that are already assigned are ignored. Tags must belong to the same workspace.

        Parameters
        ----------
        conversation_id : str

        tag_ids : typing.Sequence[str]
            Tag IDs to add to the conversation. Re-assigning an existing tag is a no-op.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.conversations.tags.assign(
                conversation_id="conversation_id",
                tag_ids=["tag_ids"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.assign(conversation_id, tag_ids=tag_ids, request_options=request_options)
        return _response.data

    async def unassign(
        self, conversation_id: str, tag_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        Remove a single conversation tag from a conversation.

        Parameters
        ----------
        conversation_id : str

        tag_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.conversations.tags.unassign(
                conversation_id="conversation_id",
                tag_id="tag_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.unassign(conversation_id, tag_id, request_options=request_options)
        return _response.data

    async def list(
        self,
        *,
        page_size: typing.Optional[int] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetConversationTagsPageResponseModel:
        """
        List conversation tags for the workspace, ordered by most recently created first.

        Parameters
        ----------
        page_size : typing.Optional[int]
            How many conversation tags to return. Can not exceed 100.

        cursor : typing.Optional[str]
            Used for fetching next page. Cursor is returned in the response.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetConversationTagsPageResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.conversations.tags.list(
                page_size=1,
                cursor="cursor",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(page_size=page_size, cursor=cursor, request_options=request_options)
        return _response.data

    async def create(
        self,
        *,
        title: str,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationTagResponseModel:
        """
        Create a new conversation tag for the workspace.

        Parameters
        ----------
        title : str
            Display title of the tag.

        description : typing.Optional[str]
            Optional free-text description.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationTagResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.conversations.tags.create(
                title="title",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(title=title, description=description, request_options=request_options)
        return _response.data

    async def get(
        self, tag_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationTagResponseModel:
        """
        Get a conversation tag by ID.

        Parameters
        ----------
        tag_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationTagResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.conversations.tags.get(
                tag_id="tag_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(tag_id, request_options=request_options)
        return _response.data

    async def delete(self, tag_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a conversation tag. Restricted to the tag owner or a workspace admin.

        Parameters
        ----------
        tag_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.conversations.tags.delete(
                tag_id="tag_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(tag_id, request_options=request_options)
        return _response.data

    async def update(
        self,
        tag_id: str,
        *,
        title: typing.Optional[str] = OMIT,
        description: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationTagResponseModel:
        """
        Update a conversation tag's title and/or description. Restricted to the tag owner or a workspace admin.

        Parameters
        ----------
        tag_id : str

        title : typing.Optional[str]
            If provided, replaces the tag title. Omit to leave unchanged.

        description : typing.Optional[str]
            If provided, replaces the tag description. Omit to leave unchanged.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationTagResponseModel
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.conversations.tags.update(
                tag_id="tag_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            tag_id, title=title, description=description, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .alignment import Alignment
from .normalized_alignment import NormalizedAlignment


class WebsocketTtsServerMessageMulti(UncheckedBaseModel):
    """
    Message sent from the server to the client for the multi-context TTS WebSocket.
    """

    audio: typing.Optional[str] = pydantic.Field(default=None)
    """
    A generated partial audio chunk, encoded using the selected output_format (e.g., MP3 as a base64 string).
    """

    is_final: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If true, indicates that this is the final message for the specified contextId. This is sent when a context is closed. `audio` will be null or empty.
    """

    normalized_alignment: typing_extensions.Annotated[
        typing.Optional[NormalizedAlignment], FieldMetadata(alias="normalizedAlignment")
    ] = None
    alignment: typing.Optional[Alignment] = None
    context_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="contextId")] = pydantic.Field(
        default=None
    )
    """
    The context identifier to which this message pertains.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .open_ai_session_config import OpenAiSessionConfig
from .realtime_config_snapshot_parents import RealtimeConfigSnapshotParents
from .safety_common_model_output import SafetyCommonModelOutput


class RealtimeConfigSnapshot(UncheckedBaseModel):
    id: typing_extensions.Annotated[str, FieldMetadata(alias="_id")]
    workspace_id: str
    conversation_id: str
    seq_no: int
    time_in_call_secs: int
    time_committed_secs: int
    source_event_id: typing.Optional[int] = None
    parents: RealtimeConfigSnapshotParents
    session_config: OpenAiSessionConfig
    safety: SafetyCommonModelOutput
    changed_paths: typing.Optional[typing.List[str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.conversation_initiation_client_data_request_input import ConversationInitiationClientDataRequestInput
from ...types.whats_app_outbound_call_response import WhatsAppOutboundCallResponse
from ...types.whats_app_outbound_message_response import WhatsAppOutboundMessageResponse
from .types.body_send_an_outbound_message_via_whats_app_v_1_convai_whatsapp_outbound_message_post_template_params_item import (
    BodySendAnOutboundMessageViaWhatsAppV1ConvaiWhatsappOutboundMessagePostTemplateParamsItem,
)

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawWhatsappClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def outbound_call(
        self,
        *,
        whatsapp_phone_number_id: str,
        whatsapp_user_id: str,
        whatsapp_call_permission_request_template_name: str,
        whatsapp_call_permission_request_template_language_code: str,
        agent_id: str,
        conversation_initiation_client_data: typing.Optional[ConversationInitiationClientDataRequestInput] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[WhatsAppOutboundCallResponse]:
        """
        Make an outbound call via WhatsApp

        Parameters
        ----------
        whatsapp_phone_number_id : str

        whatsapp_user_id : str

        whatsapp_call_permission_request_template_name : str

        whatsapp_call_permission_request_template_language_code : str

        agent_id : str

        conversation_initiation_client_data : typing.Optional[ConversationInitiationClientDataRequestInput]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[WhatsAppOutboundCallResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/convai/whatsapp/outbound-call",
            method="POST",
            json={
                "whatsapp_phone_number_id": whatsapp_phone_number_id,
                "whatsapp_user_id": whatsapp_user_id,
                "whatsapp_call_permission_request_template_name": whatsapp_call_permission_request_template_name,
                "whatsapp_call_permission_request_template_language_code": whatsapp_call_permission_request_template_language_code,
                "agent_id": agent_id,
                "conversation_initiation_client_data": convert_and_respect_annotation_metadata(
                    object_=conversation_initiation_client_data,
                    annotation=ConversationInitiationClientDataRequestInput,
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WhatsAppOutboundCallResponse,
                    construct_type(
                        type_=WhatsAppOutboundCallResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def outbound_message(
        self,
        *,
        whatsapp_phone_number_id: str,
        whatsapp_user_id: str,
        template_name: str,
        template_language_code: str,
        template_params: typing.Sequence[
            BodySendAnOutboundMessageViaWhatsAppV1ConvaiWhatsappOutboundMessagePostTemplateParamsItem
        ],
        agent_id: str,
        conversation_initiation_client_data: typing.Optional[ConversationInitiationClientDataRequestInput] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[WhatsAppOutboundMessageResponse]:
        """
        Send an outbound message via WhatsApp

        Parameters
        ----------
        whatsapp_phone_number_id : str

        whatsapp_user_id : str

        template_name : str

        template_language_code : str

        template_params : typing.Sequence[BodySendAnOutboundMessageViaWhatsAppV1ConvaiWhatsappOutboundMessagePostTemplateParamsItem]

        agent_id : str

        conversation_initiation_client_data : typing.Optional[ConversationInitiationClientDataRequestInput]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[WhatsAppOutboundMessageResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v1/convai/whatsapp/outbound-message",
            method="POST",
            json={
                "whatsapp_phone_number_id": whatsapp_phone_number_id,
                "whatsapp_user_id": whatsapp_user_id,
                "template_name": template_name,
                "template_language_code": template_language_code,
                "template_params": convert_and_respect_annotation_metadata(
                    object_=template_params,
                    annotation=typing.Sequence[
                        BodySendAnOutboundMessageViaWhatsAppV1ConvaiWhatsappOutboundMessagePostTemplateParamsItem
                    ],
                    direction="write",
                ),
                "agent_id": agent_id,
                "conversation_initiation_client_data": convert_and_respect_annotation_metadata(
                    object_=conversation_initiation_client_data,
                    annotation=ConversationInitiationClientDataRequestInput,
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WhatsAppOutboundMessageResponse,
                    construct_type(
                        type_=WhatsAppOutboundMessageResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawWhatsappClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def outbound_call(
        self,
        *,
        whatsapp_phone_number_id: str,
        whatsapp_user_id: str,
        whatsapp_call_permission_request_template_name: str,
        whatsapp_call_permission_request_template_language_code: str,
        agent_id: str,
        conversation_initiation_client_data: typing.Optional[ConversationInitiationClientDataRequestInput] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[WhatsAppOutboundCallResponse]:
        """
        Make an outbound call via WhatsApp

        Parameters
        ----------
        whatsapp_phone_number_id : str

        whatsapp_user_id : str

        whatsapp_call_permission_request_template_name : str

        whatsapp_call_permission_request_template_language_code : str

        agent_id : str

        conversation_initiation_client_data : typing.Optional[ConversationInitiationClientDataRequestInput]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[WhatsAppOutboundCallResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/convai/whatsapp/outbound-call",
            method="POST",
            json={
                "whatsapp_phone_number_id": whatsapp_phone_number_id,
                "whatsapp_user_id": whatsapp_user_id,
                "whatsapp_call_permission_request_template_name": whatsapp_call_permission_request_template_name,
                "whatsapp_call_permission_request_template_language_code": whatsapp_call_permission_request_template_language_code,
                "agent_id": agent_id,
                "conversation_initiation_client_data": convert_and_respect_annotation_metadata(
                    object_=conversation_initiation_client_data,
                    annotation=ConversationInitiationClientDataRequestInput,
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WhatsAppOutboundCallResponse,
                    construct_type(
                        type_=WhatsAppOutboundCallResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def outbound_message(
        self,
        *,
        whatsapp_phone_number_id: str,
        whatsapp_user_id: str,
        template_name: str,
        template_language_code: str,
        template_params: typing.Sequence[
            BodySendAnOutboundMessageViaWhatsAppV1ConvaiWhatsappOutboundMessagePostTemplateParamsItem
        ],
        agent_id: str,
        conversation_initiation_client_data: typing.Optional[ConversationInitiationClientDataRequestInput] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[WhatsAppOutboundMessageResponse]:
        """
        Send an outbound message via WhatsApp

        Parameters
        ----------
        whatsapp_phone_number_id : str

        whatsapp_user_id : str

        template_name : str

        template_language_code : str

        template_params : typing.Sequence[BodySendAnOutboundMessageViaWhatsAppV1ConvaiWhatsappOutboundMessagePostTemplateParamsItem]

        agent_id : str

        conversation_initiation_client_data : typing.Optional[ConversationInitiationClientDataRequestInput]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[WhatsAppOutboundMessageResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v1/convai/whatsapp/outbound-message",
            method="POST",
            json={
                "whatsapp_phone_number_id": whatsapp_phone_number_id,
                "whatsapp_user_id": whatsapp_user_id,
                "template_name": template_name,
                "template_language_code": template_language_code,
                "template_params": convert_and_respect_annotation_metadata(
                    object_=template_params,
                    annotation=typing.Sequence[
                        BodySendAnOutboundMessageViaWhatsAppV1ConvaiWhatsappOutboundMessagePostTemplateParamsItem
                    ],
                    direction="write",
                ),
                "agent_id": agent_id,
                "conversation_initiation_client_data": convert_and_respect_annotation_metadata(
                    object_=conversation_initiation_client_data,
                    annotation=ConversationInitiationClientDataRequestInput,
                    direction="write",
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    WhatsAppOutboundMessageResponse,
                    construct_type(
                        type_=WhatsAppOutboundMessageResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import os
import typing

import httpx
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .core.request_options import RequestOptions
from .environment import ElevenLabsEnvironment
from .raw_base_client import AsyncRawBaseElevenLabs, RawBaseElevenLabs

if typing.TYPE_CHECKING:
    from .audio_isolation.client import AsyncAudioIsolationClient, AudioIsolationClient
    from .audio_native.client import AsyncAudioNativeClient, AudioNativeClient
    from .conversational_ai.client import AsyncConversationalAiClient, ConversationalAiClient
    from .dubbing.client import AsyncDubbingClient, DubbingClient
    from .environment_variables.client import AsyncEnvironmentVariablesClient, EnvironmentVariablesClient
    from .forced_alignment.client import AsyncForcedAlignmentClient, ForcedAlignmentClient
    from .history.client import AsyncHistoryClient, HistoryClient
    from .models.client import AsyncModelsClient, ModelsClient
    from .music.client import AsyncMusicClient, MusicClient
    from .productions.client import AsyncProductionsClient, ProductionsClient
    from .pronunciation_dictionaries.client import AsyncPronunciationDictionariesClient, PronunciationDictionariesClient
    from .samples.client import AsyncSamplesClient, SamplesClient
    from .service_accounts.client import AsyncServiceAccountsClient, ServiceAccountsClient
    from .speech_engine.client import AsyncSpeechEngineClient, SpeechEngineClient
    from .speech_to_speech.client import AsyncSpeechToSpeechClient, SpeechToSpeechClient
    from .speech_to_text.client import AsyncSpeechToTextClient, SpeechToTextClient
    from .studio.client import AsyncStudioClient, StudioClient
    from .text_to_dialogue.client import AsyncTextToDialogueClient, TextToDialogueClient
    from .text_to_sound_effects.client import AsyncTextToSoundEffectsClient, TextToSoundEffectsClient
    from .text_to_speech.client import AsyncTextToSpeechClient, TextToSpeechClient
    from .text_to_voice.client import AsyncTextToVoiceClient, TextToVoiceClient
    from .tokens.client import AsyncTokensClient, TokensClient
    from .usage.client import AsyncUsageClient, UsageClient
    from .user.client import AsyncUserClient, UserClient
    from .voices.client import AsyncVoicesClient, VoicesClient
    from .webhooks.client import AsyncWebhooksClient, WebhooksClient
    from .workspace.client import AsyncWorkspaceClient, WorkspaceClient
    from .workspaces.client import AsyncWorkspacesClient, WorkspacesClient


class BaseElevenLabs:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : ElevenLabsEnvironment
        The environment to use for requests from the client. from .environment import ElevenLabsEnvironment



        Defaults to ElevenLabsEnvironment.PRODUCTION



    api_key : typing.Optional[str]
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 240 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from elevenlabs import ElevenLabs

    client = ElevenLabs(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: ElevenLabsEnvironment = ElevenLabsEnvironment.PRODUCTION,
        api_key: typing.Optional[str] = os.getenv("ELEVENLABS_API_KEY"),
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 240 if httpx_client is None else httpx_client.timeout.read
        )
        self._client_wrapper = SyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            api_key=api_key,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self._raw_client = RawBaseElevenLabs(client_wrapper=self._client_wrapper)
        self._history: typing.Optional[HistoryClient] = None
        self._text_to_sound_effects: typing.Optional[TextToSoundEffectsClient] = None
        self._audio_isolation: typing.Optional[AudioIsolationClient] = None
        self._samples: typing.Optional[SamplesClient] = None
        self._text_to_speech: typing.Optional[TextToSpeechClient] = None
        self._text_to_dialogue: typing.Optional[TextToDialogueClient] = None
        self._speech_to_speech: typing.Optional[SpeechToSpeechClient] = None
        self._text_to_voice: typing.Optional[TextToVoiceClient] = None
        self._user: typing.Optional[UserClient] = None
        self._voices: typing.Optional[VoicesClient] = None
        self._studio: typing.Optional[StudioClient] = None
        self._music: typing.Optional[MusicClient] = None
        self._dubbing: typing.Optional[DubbingClient] = None
        self._models: typing.Optional[ModelsClient] = None
        self._audio_native: typing.Optional[AudioNativeClient] = None
        self._usage: typing.Optional[UsageClient] = None
        self._pronunciation_dictionaries: typing.Optional[PronunciationDictionariesClient] = None
        self._service_accounts: typing.Optional[ServiceAccountsClient] = None
        self._webhooks: typing.Optional[WebhooksClient] = None
        self._speech_to_text: typing.Optional[SpeechToTextClient] = None
        self._forced_alignment: typing.Optional[ForcedAlignmentClient] = None
        self._conversational_ai: typing.Optional[ConversationalAiClient] = None
        self._speech_engine: typing.Optional[SpeechEngineClient] = None
        self._environment_variables: typing.Optional[EnvironmentVariablesClient] = None
        self._productions: typing.Optional[ProductionsClient] = None
        self._tokens: typing.Optional[TokensClient] = None
        self._workspace: typing.Optional[WorkspaceClient] = None
        self._workspaces: typing.Optional[WorkspacesClient] = None

    @property
    def with_raw_response(self) -> RawBaseElevenLabs:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawBaseElevenLabs
        """
        return self._raw_client

    def save_a_voice_preview(self, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Add a generated voice to the voice library.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.save_a_voice_preview()
        """
        _response = self._raw_client.save_a_voice_preview(request_options=request_options)
        return _response.data

    @property
    def history(self):
        if self._history is None:
            from .history.client import HistoryClient  # noqa: E402

            self._history = HistoryClient(client_wrapper=self._client_wrapper)
        return self._history

    @property
    def text_to_sound_effects(self):
        if self._text_to_sound_effects is None:
            from .text_to_sound_effects.client import TextToSoundEffectsClient  # noqa: E402

            self._text_to_sound_effects = TextToSoundEffectsClient(client_wrapper=self._client_wrapper)
        return self._text_to_sound_effects

    @property
    def audio_isolation(self):
        if self._audio_isolation is None:
            from .audio_isolation.client import AudioIsolationClient  # noqa: E402

            self._audio_isolation = AudioIsolationClient(client_wrapper=self._client_wrapper)
        return self._audio_isolation

    @property
    def samples(self):
        if self._samples is None:
            from .samples.client import SamplesClient  # noqa: E402

            self._samples = SamplesClient(client_wrapper=self._client_wrapper)
        return self._samples

    @property
    def text_to_speech(self):
        if self._text_to_speech is None:
            from .text_to_speech.client import TextToSpeechClient  # noqa: E402

            self._text_to_speech = TextToSpeechClient(client_wrapper=self._client_wrapper)
        return self._text_to_speech

    @property
    def text_to_dialogue(self):
        if self._text_to_dialogue is None:
            from .text_to_dialogue.client import TextToDialogueClient  # noqa: E402

            self._text_to_dialogue = TextToDialogueClient(client_wrapper=self._client_wrapper)
        return self._text_to_dialogue

    @property
    def speech_to_speech(self):
        if self._speech_to_speech is None:
            from .speech_to_speech.client import SpeechToSpeechClient  # noqa: E402

            self._speech_to_speech = SpeechToSpeechClient(client_wrapper=self._client_wrapper)
        return self._speech_to_speech

    @property
    def text_to_voice(self):
        if self._text_to_voice is None:
            from .text_to_voice.client import TextToVoiceClient  # noqa: E402

            self._text_to_voice = TextToVoiceClient(client_wrapper=self._client_wrapper)
        return self._text_to_voice

    @property
    def user(self):
        if self._user is None:
            from .user.client import UserClient  # noqa: E402

            self._user = UserClient(client_wrapper=self._client_wrapper)
        return self._user

    @property
    def voices(self):
        if self._voices is None:
            from .voices.client import VoicesClient  # noqa: E402

            self._voices = VoicesClient(client_wrapper=self._client_wrapper)
        return self._voices

    @property
    def studio(self):
        if self._studio is None:
            from .studio.client import StudioClient  # noqa: E402

            self._studio = StudioClient(client_wrapper=self._client_wrapper)
        return self._studio

    @property
    def music(self):
        if self._music is None:
            from .music.client import MusicClient  # noqa: E402

            self._music = MusicClient(client_wrapper=self._client_wrapper)
        return self._music

    @property
    def dubbing(self):
        if self._dubbing is None:
            from .dubbing.client import DubbingClient  # noqa: E402

            self._dubbing = DubbingClient(client_wrapper=self._client_wrapper)
        return self._dubbing

    @property
    def models(self):
        if self._models is None:
            from .models.client import ModelsClient  # noqa: E402

            self._models = ModelsClient(client_wrapper=self._client_wrapper)
        return self._models

    @property
    def audio_native(self):
        if self._audio_native is None:
            from .audio_native.client import AudioNativeClient  # noqa: E402

            self._audio_native = AudioNativeClient(client_wrapper=self._client_wrapper)
        return self._audio_native

    @property
    def usage(self):
        if self._usage is None:
            from .usage.client import UsageClient  # noqa: E402

            self._usage = UsageClient(client_wrapper=self._client_wrapper)
        return self._usage

    @property
    def pronunciation_dictionaries(self):
        if self._pronunciation_dictionaries is None:
            from .pronunciation_dictionaries.client import PronunciationDictionariesClient  # noqa: E402

            self._pronunciation_dictionaries = PronunciationDictionariesClient(client_wrapper=self._client_wrapper)
        return self._pronunciation_dictionaries

    @property
    def service_accounts(self):
        if self._service_accounts is None:
            from .service_accounts.client import ServiceAccountsClient  # noqa: E402

            self._service_accounts = ServiceAccountsClient(client_wrapper=self._client_wrapper)
        return self._service_accounts

    @property
    def webhooks(self):
        if self._webhooks is None:
            from .webhooks.client import WebhooksClient  # noqa: E402

            self._webhooks = WebhooksClient(client_wrapper=self._client_wrapper)
        return self._webhooks

    @property
    def speech_to_text(self):
        if self._speech_to_text is None:
            from .speech_to_text.client import SpeechToTextClient  # noqa: E402

            self._speech_to_text = SpeechToTextClient(client_wrapper=self._client_wrapper)
        return self._speech_to_text

    @property
    def forced_alignment(self):
        if self._forced_alignment is None:
            from .forced_alignment.client import ForcedAlignmentClient  # noqa: E402

            self._forced_alignment = ForcedAlignmentClient(client_wrapper=self._client_wrapper)
        return self._forced_alignment

    @property
    def conversational_ai(self):
        if self._conversational_ai is None:
            from .conversational_ai.client import ConversationalAiClient  # noqa: E402

            self._conversational_ai = ConversationalAiClient(client_wrapper=self._client_wrapper)
        return self._conversational_ai

    @property
    def speech_engine(self):
        if self._speech_engine is None:
            from .speech_engine.client import SpeechEngineClient  # noqa: E402

            self._speech_engine = SpeechEngineClient(client_wrapper=self._client_wrapper)
        return self._speech_engine

    @property
    def environment_variables(self):
        if self._environment_variables is None:
            from .environment_variables.client import EnvironmentVariablesClient  # noqa: E402

            self._environment_variables = EnvironmentVariablesClient(client_wrapper=self._client_wrapper)
        return self._environment_variables

    @property
    def productions(self):
        if self._productions is None:
            from .productions.client import ProductionsClient  # noqa: E402

            self._productions = ProductionsClient(client_wrapper=self._client_wrapper)
        return self._productions

    @property
    def tokens(self):
        if self._tokens is None:
            from .tokens.client import TokensClient  # noqa: E402

            self._tokens = TokensClient(client_wrapper=self._client_wrapper)
        return self._tokens

    @property
    def workspace(self):
        if self._workspace is None:
            from .workspace.client import WorkspaceClient  # noqa: E402

            self._workspace = WorkspaceClient(client_wrapper=self._client_wrapper)
        return self._workspace

    @property
    def workspaces(self):
        if self._workspaces is None:
            from .workspaces.client import WorkspacesClient  # noqa: E402

            self._workspaces = WorkspacesClient(client_wrapper=self._client_wrapper)
        return self._workspaces


class AsyncBaseElevenLabs:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : ElevenLabsEnvironment
        The environment to use for requests from the client. from .environment import ElevenLabsEnvironment



        Defaults to ElevenLabsEnvironment.PRODUCTION



    api_key : typing.Optional[str]
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 240 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from elevenlabs import AsyncElevenLabs

    client = AsyncElevenLabs(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: ElevenLabsEnvironment = ElevenLabsEnvironment.PRODUCTION,
        api_key: typing.Optional[str] = os.getenv("ELEVENLABS_API_KEY"),
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 240 if httpx_client is None else httpx_client.timeout.read
        )
        self._client_wrapper = AsyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            api_key=api_key,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self._raw_client = AsyncRawBaseElevenLabs(client_wrapper=self._client_wrapper)
        self._history: typing.Optional[AsyncHistoryClient] = None
        self._text_to_sound_effects: typing.Optional[AsyncTextToSoundEffectsClient] = None
        self._audio_isolation: typing.Optional[AsyncAudioIsolationClient] = None
        self._samples: typing.Optional[AsyncSamplesClient] = None
        self._text_to_speech: typing.Optional[AsyncTextToSpeechClient] = None
        self._text_to_dialogue: typing.Optional[AsyncTextToDialogueClient] = None
        self._speech_to_speech: typing.Optional[AsyncSpeechToSpeechClient] = None
        self._text_to_voice: typing.Optional[AsyncTextToVoiceClient] = None
        self._user: typing.Optional[AsyncUserClient] = None
        self._voices: typing.Optional[AsyncVoicesClient] = None
        self._studio: typing.Optional[AsyncStudioClient] = None
        self._music: typing.Optional[AsyncMusicClient] = None
        self._dubbing: typing.Optional[AsyncDubbingClient] = None
        self._models: typing.Optional[AsyncModelsClient] = None
        self._audio_native: typing.Optional[AsyncAudioNativeClient] = None
        self._usage: typing.Optional[AsyncUsageClient] = None
        self._pronunciation_dictionaries: typing.Optional[AsyncPronunciationDictionariesClient] = None
        self._service_accounts: typing.Optional[AsyncServiceAccountsClient] = None
        self._webhooks: typing.Optional[AsyncWebhooksClient] = None
        self._speech_to_text: typing.Optional[AsyncSpeechToTextClient] = None
        self._forced_alignment: typing.Optional[AsyncForcedAlignmentClient] = None
        self._conversational_ai: typing.Optional[AsyncConversationalAiClient] = None
        self._speech_engine: typing.Optional[AsyncSpeechEngineClient] = None
        self._environment_variables: typing.Optional[AsyncEnvironmentVariablesClient] = None
        self._productions: typing.Optional[AsyncProductionsClient] = None
        self._tokens: typing.Optional[AsyncTokensClient] = None
        self._workspace: typing.Optional[AsyncWorkspaceClient] = None
        self._workspaces: typing.Optional[AsyncWorkspacesClient] = None

    @property
    def with_raw_response(self) -> AsyncRawBaseElevenLabs:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawBaseElevenLabs
        """
        return self._raw_client

    async def save_a_voice_preview(self, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Add a generated voice to the voice library.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.save_a_voice_preview()


        asyncio.run(main())
        """
        _response = await self._raw_client.save_a_voice_preview(request_options=request_options)
        return _response.data

    @property
    def history(self):
        if self._history is None:
            from .history.client import AsyncHistoryClient  # noqa: E402

            self._history = AsyncHistoryClient(client_wrapper=self._client_wrapper)
        return self._history

    @property
    def text_to_sound_effects(self):
        if self._text_to_sound_effects is None:
            from .text_to_sound_effects.client import AsyncTextToSoundEffectsClient  # noqa: E402

            self._text_to_sound_effects = AsyncTextToSoundEffectsClient(client_wrapper=self._client_wrapper)
        return self._text_to_sound_effects

    @property
    def audio_isolation(self):
        if self._audio_isolation is None:
            from .audio_isolation.client import AsyncAudioIsolationClient  # noqa: E402

            self._audio_isolation = AsyncAudioIsolationClient(client_wrapper=self._client_wrapper)
        return self._audio_isolation

    @property
    def samples(self):
        if self._samples is None:
            from .samples.client import AsyncSamplesClient  # noqa: E402

            self._samples = AsyncSamplesClient(client_wrapper=self._client_wrapper)
        return self._samples

    @property
    def text_to_speech(self):
        if self._text_to_speech is None:
            from .text_to_speech.client import AsyncTextToSpeechClient  # noqa: E402

            self._text_to_speech = AsyncTextToSpeechClient(client_wrapper=self._client_wrapper)
        return self._text_to_speech

    @property
    def text_to_dialogue(self):
        if self._text_to_dialogue is None:
            from .text_to_dialogue.client import AsyncTextToDialogueClient  # noqa: E402

            self._text_to_dialogue = AsyncTextToDialogueClient(client_wrapper=self._client_wrapper)
        return self._text_to_dialogue

    @property
    def speech_to_speech(self):
        if self._speech_to_speech is None:
            from .speech_to_speech.client import AsyncSpeechToSpeechClient  # noqa: E402

            self._speech_to_speech = AsyncSpeechToSpeechClient(client_wrapper=self._client_wrapper)
        return self._speech_to_speech

    @property
    def text_to_voice(self):
        if self._text_to_voice is None:
            from .text_to_voice.client import AsyncTextToVoiceClient  # noqa: E402

            self._text_to_voice = AsyncTextToVoiceClient(client_wrapper=self._client_wrapper)
        return self._text_to_voice

    @property
    def user(self):
        if self._user is None:
            from .user.client import AsyncUserClient  # noqa: E402

            self._user = AsyncUserClient(client_wrapper=self._client_wrapper)
        return self._user

    @property
    def voices(self):
        if self._voices is None:
            from .voices.client import AsyncVoicesClient  # noqa: E402

            self._voices = AsyncVoicesClient(client_wrapper=self._client_wrapper)
        return self._voices

    @property
    def studio(self):
        if self._studio is None:
            from .studio.client import AsyncStudioClient  # noqa: E402

            self._studio = AsyncStudioClient(client_wrapper=self._client_wrapper)
        return self._studio

    @property
    def music(self):
        if self._music is None:
            from .music.client import AsyncMusicClient  # noqa: E402

            self._music = AsyncMusicClient(client_wrapper=self._client_wrapper)
        return self._music

    @property
    def dubbing(self):
        if self._dubbing is None:
            from .dubbing.client import AsyncDubbingClient  # noqa: E402

            self._dubbing = AsyncDubbingClient(client_wrapper=self._client_wrapper)
        return self._dubbing

    @property
    def models(self):
        if self._models is None:
            from .models.client import AsyncModelsClient  # noqa: E402

            self._models = AsyncModelsClient(client_wrapper=self._client_wrapper)
        return self._models

    @property
    def audio_native(self):
        if self._audio_native is None:
            from .audio_native.client import AsyncAudioNativeClient  # noqa: E402

            self._audio_native = AsyncAudioNativeClient(client_wrapper=self._client_wrapper)
        return self._audio_native

    @property
    def usage(self):
        if self._usage is None:
            from .usage.client import AsyncUsageClient  # noqa: E402

            self._usage = AsyncUsageClient(client_wrapper=self._client_wrapper)
        return self._usage

    @property
    def pronunciation_dictionaries(self):
        if self._pronunciation_dictionaries is None:
            from .pronunciation_dictionaries.client import AsyncPronunciationDictionariesClient  # noqa: E402

            self._pronunciation_dictionaries = AsyncPronunciationDictionariesClient(client_wrapper=self._client_wrapper)
        return self._pronunciation_dictionaries

    @property
    def service_accounts(self):
        if self._service_accounts is None:
            from .service_accounts.client import AsyncServiceAccountsClient  # noqa: E402

            self._service_accounts = AsyncServiceAccountsClient(client_wrapper=self._client_wrapper)
        return self._service_accounts

    @property
    def webhooks(self):
        if self._webhooks is None:
            from .webhooks.client import AsyncWebhooksClient  # noqa: E402

            self._webhooks = AsyncWebhooksClient(client_wrapper=self._client_wrapper)
        return self._webhooks

    @property
    def speech_to_text(self):
        if self._speech_to_text is None:
            from .speech_to_text.client import AsyncSpeechToTextClient  # noqa: E402

            self._speech_to_text = AsyncSpeechToTextClient(client_wrapper=self._client_wrapper)
        return self._speech_to_text

    @property
    def forced_alignment(self):
        if self._forced_alignment is None:
            from .forced_alignment.client import AsyncForcedAlignmentClient  # noqa: E402

            self._forced_alignment = AsyncForcedAlignmentClient(client_wrapper=self._client_wrapper)
        return self._forced_alignment

    @property
    def conversational_ai(self):
        if self._conversational_ai is None:
            from .conversational_ai.client import AsyncConversationalAiClient  # noqa: E402

            self._conversational_ai = AsyncConversationalAiClient(client_wrapper=self._client_wrapper)
        return self._conversational_ai

    @property
    def speech_engine(self):
        if self._speech_engine is None:
            from .speech_engine.client import AsyncSpeechEngineClient  # noqa: E402

            self._speech_engine = AsyncSpeechEngineClient(client_wrapper=self._client_wrapper)
        return self._speech_engine

    @property
    def environment_variables(self):
        if self._environment_variables is None:
            from .environment_variables.client import AsyncEnvironmentVariablesClient  # noqa: E402

            self._environment_variables = AsyncEnvironmentVariablesClient(client_wrapper=self._client_wrapper)
        return self._environment_variables

    @property
    def productions(self):
        if self._productions is None:
            from .productions.client import AsyncProductionsClient  # noqa: E402

            self._productions = AsyncProductionsClient(client_wrapper=self._client_wrapper)
        return self._productions

    @property
    def tokens(self):
        if self._tokens is None:
            from .tokens.client import AsyncTokensClient  # noqa: E402

            self._tokens = AsyncTokensClient(client_wrapper=self._client_wrapper)
        return self._tokens

    @property
    def workspace(self):
        if self._workspace is None:
            from .workspace.client import AsyncWorkspaceClient  # noqa: E402

            self._workspace = AsyncWorkspaceClient(client_wrapper=self._client_wrapper)
        return self._workspace

    @property
    def workspaces(self):
        if self._workspaces is None:
            from .workspaces.client import AsyncWorkspacesClient  # noqa: E402

            self._workspaces = AsyncWorkspacesClient(client_wrapper=self._client_wrapper)
        return self._workspaces


def _get_base_url(*, base_url: typing.Optional[str] = None, environment: ElevenLabsEnvironment) -> str:
    if base_url is not None:
        return base_url
    elif environment is not None:
        return environment.value
    else:
        raise Exception("Please pass in either base_url or environment to construct the client")



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel


class VoiceSegment(UncheckedBaseModel):
    voice_id: str = pydantic.Field()
    """
    The voice ID used for this segment
    """

    start_time_seconds: float = pydantic.Field()
    """
    Start time of this voice segment
    """

    end_time_seconds: float = pydantic.Field()
    """
    End time of this voice segment
    """

    character_start_index: int = pydantic.Field()
    """
    Start index in the characters array
    """

    character_end_index: int = pydantic.Field()
    """
    End index in the characters array (exclusive)
    """

    dialogue_input_index: int = pydantic.Field()
    """
    Line of the dialogue (script) that this segment is a part of.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ConversationInitiationSource = typing.Union[
    typing.Literal[
        "unknown",
        "android_sdk",
        "node_js_sdk",
        "react_native_sdk",
        "react_sdk",
        "js_sdk",
        "python_sdk",
        "widget",
        "sip_trunk",
        "twilio",
        "exotel",
        "genesys",
        "swift_sdk",
        "whatsapp",
        "twilio_sms",
        "flutter_sdk",
        "zendesk_integration",
        "slack_integration",
        "telegram_integration",
        "intercom_integration",
        "freshdesk_integration",
        "template_preview",
        "genesys_bot_connector",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from ..core.unchecked_base_model import UncheckedBaseModel
from .default_sharing_group_response_model import DefaultSharingGroupResponseModel
from .workspace_api_key_response_model import WorkspaceApiKeyResponseModel


class WorkspaceServiceAccountResponseModel(UncheckedBaseModel):
    service_account_user_id: str
    name: str
    created_at_unix: typing.Optional[int] = None
    api_keys: typing_extensions.Annotated[typing.List[WorkspaceApiKeyResponseModel], FieldMetadata(alias="api-keys")]
    default_sharing_groups: typing.Optional[typing.List[DefaultSharingGroupResponseModel]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .... import core
from ....core.api_error import ApiError
from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.http_response import AsyncHttpResponse, HttpResponse
from ....core.jsonable_encoder import jsonable_encoder
from ....core.request_options import RequestOptions
from ....core.unchecked_base_model import construct_type
from ....errors.unprocessable_entity_error import UnprocessableEntityError
from ....types.media_id import MediaId
from ....types.order_id import OrderId
from ....types.order_media_response import OrderMediaResponse
from ....types.register_media_response import RegisterMediaResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawMediaClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def register(
        self,
        order_id: OrderId,
        *,
        declared_language: str,
        media: typing.Optional[core.File] = OMIT,
        media_url: typing.Optional[str] = OMIT,
        media_url_filename: typing.Optional[str] = OMIT,
        media_url_content_type: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[RegisterMediaResponse]:
        """
        Registers a media file with an order, either by uploading it directly or by providing a URL to fetch it from. Exactly one of `media` or `media_url` must be provided. The registered media can then be referenced when adding order items.

        Parameters
        ----------
        order_id : OrderId
            The ID of the order to which this media will be attached.

        declared_language : str
            The language code of the media content (e.g. 'en', 'es-ES'). Must be a supported source language for some order item kind.

        media : typing.Optional[core.File]
            See core.File for more documentation

        media_url : typing.Optional[str]
            A URL to fetch the media file from. Mutually exclusive with media.

        media_url_filename : typing.Optional[str]
            The filename for URL-sourced media (e.g. 'example.mp4'). Required when using media_url.

        media_url_content_type : typing.Optional[str]
            The MIME type for URL-sourced media (e.g. 'video/mp4'). Required when using media_url.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RegisterMediaResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/productions/orders/{jsonable_encoder(order_id)}/media",
            method="POST",
            data={
                "declared_language": declared_language,
                "media_url": media_url,
                "media_url_filename": media_url_filename,
                "media_url_content_type": media_url_content_type,
            },
            files={
                **({"media": media} if media is not None else {}),
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RegisterMediaResponse,
                    construct_type(
                        type_=RegisterMediaResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, order_id: OrderId, media_id: MediaId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[OrderMediaResponse]:
        """
        Retrieves metadata and a time-limited download URL for a previously uploaded media file.

        Parameters
        ----------
        order_id : OrderId
            The ID of the order.

        media_id : MediaId
            The ID of the media file.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[OrderMediaResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/productions/orders/{jsonable_encoder(order_id)}/media/{jsonable_encoder(media_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    OrderMediaResponse,
                    construct_type(
                        type_=OrderMediaResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawMediaClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def register(
        self,
        order_id: OrderId,
        *,
        declared_language: str,
        media: typing.Optional[core.File] = OMIT,
        media_url: typing.Optional[str] = OMIT,
        media_url_filename: typing.Optional[str] = OMIT,
        media_url_content_type: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[RegisterMediaResponse]:
        """
        Registers a media file with an order, either by uploading it directly or by providing a URL to fetch it from. Exactly one of `media` or `media_url` must be provided. The registered media can then be referenced when adding order items.

        Parameters
        ----------
        order_id : OrderId
            The ID of the order to which this media will be attached.

        declared_language : str
            The language code of the media content (e.g. 'en', 'es-ES'). Must be a supported source language for some order item kind.

        media : typing.Optional[core.File]
            See core.File for more documentation

        media_url : typing.Optional[str]
            A URL to fetch the media file from. Mutually exclusive with media.

        media_url_filename : typing.Optional[str]
            The filename for URL-sourced media (e.g. 'example.mp4'). Required when using media_url.

        media_url_content_type : typing.Optional[str]
            The MIME type for URL-sourced media (e.g. 'video/mp4'). Required when using media_url.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RegisterMediaResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/productions/orders/{jsonable_encoder(order_id)}/media",
            method="POST",
            data={
                "declared_language": declared_language,
                "media_url": media_url,
                "media_url_filename": media_url_filename,
                "media_url_content_type": media_url_content_type,
            },
            files={
                **({"media": media} if media is not None else {}),
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RegisterMediaResponse,
                    construct_type(
                        type_=RegisterMediaResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, order_id: OrderId, media_id: MediaId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[OrderMediaResponse]:
        """
        Retrieves metadata and a time-limited download URL for a previously uploaded media file.

        Parameters
        ----------
        order_id : OrderId
            The ID of the order.

        media_id : MediaId
            The ID of the media file.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[OrderMediaResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/productions/orders/{jsonable_encoder(order_id)}/media/{jsonable_encoder(media_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    OrderMediaResponse,
                    construct_type(
                        type_=OrderMediaResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.request_options import RequestOptions
from ....types.agent_workflow_request_model import AgentWorkflowRequestModel
from .raw_client import AsyncRawDraftsClient, RawDraftsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class DraftsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawDraftsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawDraftsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawDraftsClient
        """
        return self._raw_client

    def create(
        self,
        agent_id: str,
        *,
        branch_id: str,
        conversation_config: typing.Dict[str, typing.Any],
        platform_settings: typing.Dict[str, typing.Any],
        workflow: AgentWorkflowRequestModel,
        name: str,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Any:
        """
        Create a new draft for an agent

        Parameters
        ----------
        agent_id : str
            The id of an agent. This is returned on agent creation.

        branch_id : str
            The ID of the agent branch to use

        conversation_config : typing.Dict[str, typing.Any]
            Conversation config for the draft

        platform_settings : typing.Dict[str, typing.Any]
            Platform settings for the draft

        workflow : AgentWorkflowRequestModel
            Workflow for the draft

        name : str
            Name for the draft

        tags : typing.Optional[typing.Sequence[str]]
            Tags to help classify and filter the agent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Any
            Successful Response

        Examples
        --------
        from elevenlabs import (
            AgentWorkflowRequestModel,
            AgentWorkflowRequestModelNodesValue_End,
            AstNodeInput_AndOperator,
            ElevenLabs,
            WorkflowEdgeModelInput,
            WorkflowEdgeModelInputForwardCondition_Expression,
        )

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.agents.drafts.create(
            agent_id="agent_3701k3ttaq12ewp8b7qv5rfyszkz",
            branch_id="agtbrch_8901k4t9z5defmb8vh3e9361y7nj",
            conversation_config={"key": "value"},
            platform_settings={"key": "value"},
            workflow=AgentWorkflowRequestModel(
                edges={
                    "entry_to_tool_a": WorkflowEdgeModelInput(
                        source="entry_node",
                        target="tool_node_a",
                        forward_condition=WorkflowEdgeModelInputForwardCondition_Expression(
                            expression=AstNodeInput_AndOperator(
                                children=[],
                            ),
                        ),
                    ),
                    "start_to_entry": WorkflowEdgeModelInput(
                        source="start_node",
                        target="entry_node",
                        forward_condition=WorkflowEdgeModelInputForwardCondition_Expression(
                            expression=AstNodeInput_AndOperator(
                                children=[],
                            ),
                        ),
                    ),
                    "tool_a_to_failure": WorkflowEdgeModelInput(
                        source="tool_node_a",
                        target="failure_node",
                        forward_condition=WorkflowEdgeModelInputForwardCondition_Expression(
                            expression=AstNodeInput_AndOperator(
                                children=[],
                            ),
                        ),
                    ),
                    "tool_a_to_tool_b": WorkflowEdgeModelInput(
                        source="tool_node_a",
                        target="tool_node_b",
                        forward_condition=WorkflowEdgeModelInputForwardCondition_Expression(
                            expression=AstNodeInput_AndOperator(
                                children=[],
                            ),
                        ),
                    ),
                    "tool_b_to_agent_transfer": WorkflowEdgeModelInput(
                        source="tool_node_b",
                        target="success_transfer",
                        forward_condition=WorkflowEdgeModelInputForwardCondition_Expression(
                            expression=AstNodeInput_AndOperator(
                                children=[],
                            ),
                        ),
                    ),
                    "tool_b_to_conversation": WorkflowEdgeModelInput(
                        source="tool_node_b",
                        target="success_conversation",
                        forward_condition=WorkflowEdgeModelInputForwardCondition_Expression(
                            expression=AstNodeInput_AndOperator(
                                children=[],
                            ),
                        ),
                    ),
                    "tool_b_to_end": WorkflowEdgeModelInput(
                        source="tool_node_b",
                        target="success_end",
                        forward_condition=WorkflowEdgeModelInputForwardCondition_Expression(
                            expression=AstNodeInput_AndOperator(
                                children=[],
                            ),
                        ),
                    ),
                    "tool_b_to_phone": WorkflowEdgeModelInput(
                        source="tool_node_b",
                        target="success_phone",
                        forward_condition=WorkflowEdgeModelInputForwardCondition_Expression(
                            expression=AstNodeInput_AndOperator(
                                children=[],
                            ),
                        ),
                    ),
                },
                nodes={
                    "entry_node": AgentWorkflowRequestModelNodesValue_End(),
                    "failure_node": AgentWorkflowRequestModelNodesValue_End(),
                    "start_node": AgentWorkflowRequestModelNodesValue_End(),
                    "success_conversation": AgentWorkflowRequestModelNodesValue_End(),
                    "success_end": AgentWorkflowRequestModelNodesValue_End(),
                    "success_phone": AgentWorkflowRequestModelNodesValue_End(),
                    "success_transfer": AgentWorkflowRequestModelNodesValue_End(),
                    "tool_node_a": AgentWorkflowRequestModelNodesValue_End(),
                    "tool_node_b": AgentWorkflowRequestModelNodesValue_End(),
                },
            ),
            name="name",
        )
        """
        _response = self._raw_client.create(
            agent_id,
            branch_id=branch_id,
            conversation_config=conversation_config,
            platform_settings=platform_settings,
            workflow=workflow,
            name=name,
            tags=tags,
            request_options=request_options,
        )
        return _response.data

    def delete(
        self, agent_id: str, *, branch_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Any:
        """
        Delete a draft for an agent

        Parameters
        ----------
        agent_id : str
            The id of an agent. This is returned on agent creation.

        branch_id : str
            The ID of the agent branch to use

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Any
            Successful Response

        Examples
        --------
        from elevenlabs import ElevenLabs

        client = ElevenLabs(
            api_key="YOUR_API_KEY",
        )
        client.conversational_ai.agents.drafts.delete(
            agent_id="agent_3701k3ttaq12ewp8b7qv5rfyszkz",
            branch_id="agtbrch_8901k4t9z5defmb8vh3e9361y7nj",
        )
        """
        _response = self._raw_client.delete(agent_id, branch_id=branch_id, request_options=request_options)
        return _response.data


class AsyncDraftsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawDraftsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawDraftsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawDraftsClient
        """
        return self._raw_client

    async def create(
        self,
        agent_id: str,
        *,
        branch_id: str,
        conversation_config: typing.Dict[str, typing.Any],
        platform_settings: typing.Dict[str, typing.Any],
        workflow: AgentWorkflowRequestModel,
        name: str,
        tags: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Any:
        """
        Create a new draft for an agent

        Parameters
        ----------
        agent_id : str
            The id of an agent. This is returned on agent creation.

        branch_id : str
            The ID of the agent branch to use

        conversation_config : typing.Dict[str, typing.Any]
            Conversation config for the draft

        platform_settings : typing.Dict[str, typing.Any]
            Platform settings for the draft

        workflow : AgentWorkflowRequestModel
            Workflow for the draft

        name : str
            Name for the draft

        tags : typing.Optional[typing.Sequence[str]]
            Tags to help classify and filter the agent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Any
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import (
            AgentWorkflowRequestModel,
            AgentWorkflowRequestModelNodesValue_End,
            AstNodeInput_AndOperator,
            AsyncElevenLabs,
            WorkflowEdgeModelInput,
            WorkflowEdgeModelInputForwardCondition_Expression,
        )

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.agents.drafts.create(
                agent_id="agent_3701k3ttaq12ewp8b7qv5rfyszkz",
                branch_id="agtbrch_8901k4t9z5defmb8vh3e9361y7nj",
                conversation_config={"key": "value"},
                platform_settings={"key": "value"},
                workflow=AgentWorkflowRequestModel(
                    edges={
                        "entry_to_tool_a": WorkflowEdgeModelInput(
                            source="entry_node",
                            target="tool_node_a",
                            forward_condition=WorkflowEdgeModelInputForwardCondition_Expression(
                                expression=AstNodeInput_AndOperator(
                                    children=[],
                                ),
                            ),
                        ),
                        "start_to_entry": WorkflowEdgeModelInput(
                            source="start_node",
                            target="entry_node",
                            forward_condition=WorkflowEdgeModelInputForwardCondition_Expression(
                                expression=AstNodeInput_AndOperator(
                                    children=[],
                                ),
                            ),
                        ),
                        "tool_a_to_failure": WorkflowEdgeModelInput(
                            source="tool_node_a",
                            target="failure_node",
                            forward_condition=WorkflowEdgeModelInputForwardCondition_Expression(
                                expression=AstNodeInput_AndOperator(
                                    children=[],
                                ),
                            ),
                        ),
                        "tool_a_to_tool_b": WorkflowEdgeModelInput(
                            source="tool_node_a",
                            target="tool_node_b",
                            forward_condition=WorkflowEdgeModelInputForwardCondition_Expression(
                                expression=AstNodeInput_AndOperator(
                                    children=[],
                                ),
                            ),
                        ),
                        "tool_b_to_agent_transfer": WorkflowEdgeModelInput(
                            source="tool_node_b",
                            target="success_transfer",
                            forward_condition=WorkflowEdgeModelInputForwardCondition_Expression(
                                expression=AstNodeInput_AndOperator(
                                    children=[],
                                ),
                            ),
                        ),
                        "tool_b_to_conversation": WorkflowEdgeModelInput(
                            source="tool_node_b",
                            target="success_conversation",
                            forward_condition=WorkflowEdgeModelInputForwardCondition_Expression(
                                expression=AstNodeInput_AndOperator(
                                    children=[],
                                ),
                            ),
                        ),
                        "tool_b_to_end": WorkflowEdgeModelInput(
                            source="tool_node_b",
                            target="success_end",
                            forward_condition=WorkflowEdgeModelInputForwardCondition_Expression(
                                expression=AstNodeInput_AndOperator(
                                    children=[],
                                ),
                            ),
                        ),
                        "tool_b_to_phone": WorkflowEdgeModelInput(
                            source="tool_node_b",
                            target="success_phone",
                            forward_condition=WorkflowEdgeModelInputForwardCondition_Expression(
                                expression=AstNodeInput_AndOperator(
                                    children=[],
                                ),
                            ),
                        ),
                    },
                    nodes={
                        "entry_node": AgentWorkflowRequestModelNodesValue_End(),
                        "failure_node": AgentWorkflowRequestModelNodesValue_End(),
                        "start_node": AgentWorkflowRequestModelNodesValue_End(),
                        "success_conversation": AgentWorkflowRequestModelNodesValue_End(),
                        "success_end": AgentWorkflowRequestModelNodesValue_End(),
                        "success_phone": AgentWorkflowRequestModelNodesValue_End(),
                        "success_transfer": AgentWorkflowRequestModelNodesValue_End(),
                        "tool_node_a": AgentWorkflowRequestModelNodesValue_End(),
                        "tool_node_b": AgentWorkflowRequestModelNodesValue_End(),
                    },
                ),
                name="name",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            agent_id,
            branch_id=branch_id,
            conversation_config=conversation_config,
            platform_settings=platform_settings,
            workflow=workflow,
            name=name,
            tags=tags,
            request_options=request_options,
        )
        return _response.data

    async def delete(
        self, agent_id: str, *, branch_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Any:
        """
        Delete a draft for an agent

        Parameters
        ----------
        agent_id : str
            The id of an agent. This is returned on agent creation.

        branch_id : str
            The ID of the agent branch to use

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.Any
            Successful Response

        Examples
        --------
        import asyncio

        from elevenlabs import AsyncElevenLabs

        client = AsyncElevenLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversational_ai.agents.drafts.delete(
                agent_id="agent_3701k3ttaq12ewp8b7qv5rfyszkz",
                branch_id="agtbrch_8901k4t9z5defmb8vh3e9361y7nj",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(agent_id, branch_id=branch_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ....core.unchecked_base_model import UncheckedBaseModel, UnionMetadata
from ....types.update_o_auth_2_jwt_request_algorithm import UpdateOAuth2JwtRequestAlgorithm
from ....types.update_o_auth_2_jwt_request_token_response_field import UpdateOAuth2JwtRequestTokenResponseField


class AuthConnectionsUpdateRequestBody_Oauth2ClientCredentials(UncheckedBaseModel):
    """
    Updated auth connection fields
    """

    auth_type: typing.Literal["oauth2_client_credentials"] = "oauth2_client_credentials"
    provider: typing.Optional[str] = None
    client_id: typing.Optional[str] = None
    scopes: typing.Optional[typing.List[str]] = None
    extra_params: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None
    basic_auth_in_header: typing.Optional[bool] = None
    client_secret: typing.Optional[str] = None
    custom_headers: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class AuthConnectionsUpdateRequestBody_BasicAuth(UncheckedBaseModel):
    """
    Updated auth connection fields
    """

    auth_type: typing.Literal["basic_auth"] = "basic_auth"
    provider: typing.Optional[str] = None
    username: typing.Optional[str] = None
    password: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class AuthConnectionsUpdateRequestBody_Oauth2Jwt(UncheckedBaseModel):
    """
    Updated auth connection fields
    """

    auth_type: typing.Literal["oauth2_jwt"] = "oauth2_jwt"
    provider: typing.Optional[str] = None
    algorithm: typing.Optional[UpdateOAuth2JwtRequestAlgorithm] = None
    key_id: typing.Optional[str] = None
    issuer: typing.Optional[str] = None
    audience: typing.Optional[str] = None
    subject: typing.Optional[str] = None
    expiration_seconds: typing.Optional[int] = None
    extra_params: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None
    scopes: typing.Optional[typing.List[str]] = None
    token_response_field: typing.Optional[UpdateOAuth2JwtRequestTokenResponseField] = None
    secret_key: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


AuthConnectionsUpdateRequestBody = typing_extensions.Annotated[
    typing.Union[
        AuthConnectionsUpdateRequestBody_Oauth2ClientCredentials,
        AuthConnectionsUpdateRequestBody_BasicAuth,
        AuthConnectionsUpdateRequestBody_Oauth2Jwt,
    ],
    UnionMetadata(discriminant="auth_type"),
]



import urllib.parse
from typing import Any, Sequence, Tuple, Union


_WS_SCHEME = {"https": "wss", "http": "ws"}


def build_ws_url(
    base_url: str,
    path_segments: Sequence[Any],
    params: Union[Sequence[Tuple[str, str]], dict],
) -> str:
    """Build a WebSocket URL with proper percent-encoding.

    Converts http(s) schemes to ws(s), appends percent-encoded
    *path_segments* beneath the existing base path, and encodes
    *params* as the query string.
    """
    parsed = urllib.parse.urlparse(base_url)
    path = "/".join(urllib.parse.quote(str(seg), safe="") for seg in path_segments)
    return urllib.parse.urlunparse((
        _WS_SCHEME.get(parsed.scheme, parsed.scheme),
        parsed.netloc,
        parsed.path.rstrip("/") + "/" + path,
        "",
        urllib.parse.urlencode(params, quote_via=urllib.parse.quote),
        "",
    ))



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .item_id import ItemId
from .order_item_request_input import OrderItemRequestInput


class UpsertOrderItemRequest(UncheckedBaseModel):
    item: OrderItemRequestInput = pydantic.Field()
    """
    The order item to add or update.
    """

    item_id: typing.Optional[ItemId] = pydantic.Field(default=None)
    """
    The ID of an existing item to update. Omit to create a new item.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

ProjectCreationMetaResponseModelStatus = typing.Union[
    typing.Literal["pending", "creating", "finished", "failed"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ....core.api_error import ApiError
from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.http_response import AsyncHttpResponse, HttpResponse
from ....core.jsonable_encoder import jsonable_encoder
from ....core.request_options import RequestOptions
from ....core.serialization import convert_and_respect_annotation_metadata
from ....core.unchecked_base_model import construct_type
from ....errors.unprocessable_entity_error import UnprocessableEntityError
from ....types.agent_branch_response import AgentBranchResponse
from ....types.agent_workflow_request_model import AgentWorkflowRequestModel
from ....types.branch_protection_status import BranchProtectionStatus
from ....types.create_agent_branch_response_model import CreateAgentBranchResponseModel
from ....types.list_response_agent_branch_summary import ListResponseAgentBranchSummary

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawBranchesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        agent_id: str,
        *,
        include_archived: typing.Optional[bool] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ListResponseAgentBranchSummary]:
        """
        Returns a list of branches an agent has

        Parameters
        ----------
        agent_id : str
            The id of an agent. This is returned on agent creation.

        include_archived : typing.Optional[bool]
            Whether archived branches should be included

        limit : typing.Optional[int]
            How many results at most should be returned

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListResponseAgentBranchSummary]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/convai/agents/{jsonable_encoder(agent_id)}/branches",
            method="GET",
            params={
                "include_archived": include_archived,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListResponseAgentBranchSummary,
                    construct_type(
                        type_=ListResponseAgentBranchSummary,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        agent_id: str,
        *,
        parent_version_id: str,
        name: str,
        description: str,
        conversation_config: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        platform_settings: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        workflow: typing.Optional[AgentWorkflowRequestModel] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateAgentBranchResponseModel]:
        """
        Create a new branch from a given version of any branch

        Parameters
        ----------
        agent_id : str
            The id of an agent. This is returned on agent creation.

        parent_version_id : str
            ID of the version to branch from

        name : str
            Name of the branch. It is unique within the agent.

        description : str
            Description for the branch

        conversation_config : typing.Optional[typing.Dict[str, typing.Any]]
            Changes to apply to conversation config

        platform_settings : typing.Optional[typing.Dict[str, typing.Any]]
            Changes to apply to platform settings

        workflow : typing.Optional[AgentWorkflowRequestModel]
            Updated workflow definition

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateAgentBranchResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/convai/agents/{jsonable_encoder(agent_id)}/branches",
            method="POST",
            json={
                "parent_version_id": parent_version_id,
                "name": name,
                "description": description,
                "conversation_config": conversation_config,
                "platform_settings": platform_settings,
                "workflow": convert_and_respect_annotation_metadata(
                    object_=workflow, annotation=AgentWorkflowRequestModel, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateAgentBranchResponseModel,
                    construct_type(
                        type_=CreateAgentBranchResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, agent_id: str, branch_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[AgentBranchResponse]:
        """
        Get information about a single agent branch

        Parameters
        ----------
        agent_id : str
            The id of an agent. This is returned on agent creation.

        branch_id : str
            Unique identifier for the branch.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AgentBranchResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/convai/agents/{jsonable_encoder(agent_id)}/branches/{jsonable_encoder(branch_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentBranchResponse,
                    construct_type(
                        type_=AgentBranchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        agent_id: str,
        branch_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        is_archived: typing.Optional[bool] = OMIT,
        protection_status: typing.Optional[BranchProtectionStatus] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[AgentBranchResponse]:
        """
        Update agent branch properties such as archiving status and protection level

        Parameters
        ----------
        agent_id : str
            The id of an agent. This is returned on agent creation.

        branch_id : str
            Unique identifier for the branch.

        name : typing.Optional[str]
            New name for the branch. Must be unique within the agent.

        is_archived : typing.Optional[bool]
            Whether the branch should be archived

        protection_status : typing.Optional[BranchProtectionStatus]
            The protection level for the branch

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[AgentBranchResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/convai/agents/{jsonable_encoder(agent_id)}/branches/{jsonable_encoder(branch_id)}",
            method="PATCH",
            json={
                "name": name,
                "is_archived": is_archived,
                "protection_status": protection_status,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentBranchResponse,
                    construct_type(
                        type_=AgentBranchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def merge(
        self,
        agent_id: str,
        source_branch_id: str,
        *,
        target_branch_id: str,
        archive_source_branch: typing.Optional[bool] = OMIT,
        force: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.Any]:
        """
        Merge a branch into a target branch

        Parameters
        ----------
        agent_id : str
            The id of an agent. This is returned on agent creation.

        source_branch_id : str
            Unique identifier for the source branch to merge from.

        target_branch_id : str
            The ID of the target branch to merge into.

        archive_source_branch : typing.Optional[bool]
            Whether to archive the source branch after merging

        force : typing.Optional[bool]
            Force source branch changes onto the target, overriding timestamp-based conflict resolution

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Any]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/convai/agents/{jsonable_encoder(agent_id)}/branches/{jsonable_encoder(source_branch_id)}/merge",
            method="POST",
            params={
                "target_branch_id": target_branch_id,
            },
            json={
                "archive_source_branch": archive_source_branch,
                "force": force,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if _response is None or not _response.text.strip():
                return HttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Any,
                    construct_type(
                        type_=typing.Any,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def rebase(
        self, agent_id: str, branch_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[typing.Any]:
        """
        Rebase a branch onto the latest main branch, incorporating main's changes while preserving the branch's own changes.

        Parameters
        ----------
        agent_id : str
            The id of an agent. This is returned on agent creation.

        branch_id : str
            Unique identifier for the source branch to merge from.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.Any]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/convai/agents/{jsonable_encoder(agent_id)}/branches/{jsonable_encoder(branch_id)}/rebase",
            method="POST",
            request_options=request_options,
        )
        try:
            if _response is None or not _response.text.strip():
                return HttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Any,
                    construct_type(
                        type_=typing.Any,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawBranchesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        agent_id: str,
        *,
        include_archived: typing.Optional[bool] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ListResponseAgentBranchSummary]:
        """
        Returns a list of branches an agent has

        Parameters
        ----------
        agent_id : str
            The id of an agent. This is returned on agent creation.

        include_archived : typing.Optional[bool]
            Whether archived branches should be included

        limit : typing.Optional[int]
            How many results at most should be returned

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListResponseAgentBranchSummary]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/convai/agents/{jsonable_encoder(agent_id)}/branches",
            method="GET",
            params={
                "include_archived": include_archived,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListResponseAgentBranchSummary,
                    construct_type(
                        type_=ListResponseAgentBranchSummary,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        agent_id: str,
        *,
        parent_version_id: str,
        name: str,
        description: str,
        conversation_config: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        platform_settings: typing.Optional[typing.Dict[str, typing.Any]] = OMIT,
        workflow: typing.Optional[AgentWorkflowRequestModel] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateAgentBranchResponseModel]:
        """
        Create a new branch from a given version of any branch

        Parameters
        ----------
        agent_id : str
            The id of an agent. This is returned on agent creation.

        parent_version_id : str
            ID of the version to branch from

        name : str
            Name of the branch. It is unique within the agent.

        description : str
            Description for the branch

        conversation_config : typing.Optional[typing.Dict[str, typing.Any]]
            Changes to apply to conversation config

        platform_settings : typing.Optional[typing.Dict[str, typing.Any]]
            Changes to apply to platform settings

        workflow : typing.Optional[AgentWorkflowRequestModel]
            Updated workflow definition

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateAgentBranchResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/convai/agents/{jsonable_encoder(agent_id)}/branches",
            method="POST",
            json={
                "parent_version_id": parent_version_id,
                "name": name,
                "description": description,
                "conversation_config": conversation_config,
                "platform_settings": platform_settings,
                "workflow": convert_and_respect_annotation_metadata(
                    object_=workflow, annotation=AgentWorkflowRequestModel, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateAgentBranchResponseModel,
                    construct_type(
                        type_=CreateAgentBranchResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, agent_id: str, branch_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[AgentBranchResponse]:
        """
        Get information about a single agent branch

        Parameters
        ----------
        agent_id : str
            The id of an agent. This is returned on agent creation.

        branch_id : str
            Unique identifier for the branch.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AgentBranchResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/convai/agents/{jsonable_encoder(agent_id)}/branches/{jsonable_encoder(branch_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentBranchResponse,
                    construct_type(
                        type_=AgentBranchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        agent_id: str,
        branch_id: str,
        *,
        name: typing.Optional[str] = OMIT,
        is_archived: typing.Optional[bool] = OMIT,
        protection_status: typing.Optional[BranchProtectionStatus] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[AgentBranchResponse]:
        """
        Update agent branch properties such as archiving status and protection level

        Parameters
        ----------
        agent_id : str
            The id of an agent. This is returned on agent creation.

        branch_id : str
            Unique identifier for the branch.

        name : typing.Optional[str]
            New name for the branch. Must be unique within the agent.

        is_archived : typing.Optional[bool]
            Whether the branch should be archived

        protection_status : typing.Optional[BranchProtectionStatus]
            The protection level for the branch

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[AgentBranchResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/convai/agents/{jsonable_encoder(agent_id)}/branches/{jsonable_encoder(branch_id)}",
            method="PATCH",
            json={
                "name": name,
                "is_archived": is_archived,
                "protection_status": protection_status,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    AgentBranchResponse,
                    construct_type(
                        type_=AgentBranchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def merge(
        self,
        agent_id: str,
        source_branch_id: str,
        *,
        target_branch_id: str,
        archive_source_branch: typing.Optional[bool] = OMIT,
        force: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.Any]:
        """
        Merge a branch into a target branch

        Parameters
        ----------
        agent_id : str
            The id of an agent. This is returned on agent creation.

        source_branch_id : str
            Unique identifier for the source branch to merge from.

        target_branch_id : str
            The ID of the target branch to merge into.

        archive_source_branch : typing.Optional[bool]
            Whether to archive the source branch after merging

        force : typing.Optional[bool]
            Force source branch changes onto the target, overriding timestamp-based conflict resolution

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Any]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/convai/agents/{jsonable_encoder(agent_id)}/branches/{jsonable_encoder(source_branch_id)}/merge",
            method="POST",
            params={
                "target_branch_id": target_branch_id,
            },
            json={
                "archive_source_branch": archive_source_branch,
                "force": force,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if _response is None or not _response.text.strip():
                return AsyncHttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Any,
                    construct_type(
                        type_=typing.Any,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def rebase(
        self, agent_id: str, branch_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[typing.Any]:
        """
        Rebase a branch onto the latest main branch, incorporating main's changes while preserving the branch's own changes.

        Parameters
        ----------
        agent_id : str
            The id of an agent. This is returned on agent creation.

        branch_id : str
            Unique identifier for the source branch to merge from.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.Any]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/convai/agents/{jsonable_encoder(agent_id)}/branches/{jsonable_encoder(branch_id)}/rebase",
            method="POST",
            request_options=request_options,
        )
        try:
            if _response is None or not _response.text.strip():
                return AsyncHttpResponse(response=_response, data=None)
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.Any,
                    construct_type(
                        type_=typing.Any,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, update_forward_refs
from ..core.unchecked_base_model import UncheckedBaseModel
from .conversation_history_transcript_common_model_output import ConversationHistoryTranscriptCommonModelOutput
from .conversation_initiation_source import ConversationInitiationSource
from .dynamic_variable_value_type_output import DynamicVariableValueTypeOutput
from .test_from_conversation_metadata_output import TestFromConversationMetadataOutput
from .unit_test_tool_call_evaluation_model_output import UnitTestToolCallEvaluationModelOutput


class GetToolCallUnitTestResponseModel(UncheckedBaseModel):
    from_conversation_metadata: typing.Optional[TestFromConversationMetadataOutput] = pydantic.Field(default=None)
    """
    Metadata of a conversation this test was created from (if applicable).
    """

    dynamic_variables: typing.Optional[typing.Dict[str, typing.Optional[DynamicVariableValueTypeOutput]]] = (
        pydantic.Field(default=None)
    )
    """
    Dynamic variables to replace in the agent config during testing
    """

    chat_history: typing.Optional[typing.List[ConversationHistoryTranscriptCommonModelOutput]] = None
    conversation_initiation_source: typing.Optional[ConversationInitiationSource] = pydantic.Field(default=None)
    """
    Simulate the test as if the conversation originated from this channel.
    """

    tool_call_parameters: typing.Optional[UnitTestToolCallEvaluationModelOutput] = pydantic.Field(default=None)
    """
    How to evaluate the agent's tool call (if any). If empty, the tool call is not evaluated.
    """

    check_any_tool_matches: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If set to True this test will pass if any tool call returned by the LLM matches the criteria. Otherwise it will fail if more than one tool is returned by the agent.
    """

    id: str
    name: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


update_forward_refs(GetToolCallUnitTestResponseModel)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .... import core
from ....core.api_error import ApiError
from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.http_response import AsyncHttpResponse, HttpResponse
from ....core.jsonable_encoder import jsonable_encoder
from ....core.request_options import RequestOptions
from ....core.unchecked_base_model import construct_type
from ....errors.unprocessable_entity_error import UnprocessableEntityError
from ....types.embedding_model_enum import EmbeddingModelEnum
from ....types.rag_document_index_response_model import RagDocumentIndexResponseModel
from .types.document_refresh_response import DocumentRefreshResponse
from .types.document_update_file_response import DocumentUpdateFileResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawDocumentClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def update_file(
        self, documentation_id: str, *, file: core.File, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DocumentUpdateFileResponse]:
        """
        Update the source file of a file document. The document name, content, and metadata are updated to reflect the new file. Any manual content edits will be overwritten.

        Parameters
        ----------
        documentation_id : str
            The id of a document from the knowledge base. This is returned on document addition.

        file : core.File
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DocumentUpdateFileResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/convai/knowledge-base/{jsonable_encoder(documentation_id)}/update-file",
            method="PATCH",
            data={},
            files={
                "file": file,
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DocumentUpdateFileResponse,
                    construct_type(
                        type_=DocumentUpdateFileResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def refresh(
        self, documentation_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DocumentRefreshResponse]:
        """
        Manually refresh a URL document by re-fetching its content from the source URL.

        Parameters
        ----------
        documentation_id : str
            The id of a document from the knowledge base. This is returned on document addition.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DocumentRefreshResponse]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/convai/knowledge-base/{jsonable_encoder(documentation_id)}/refresh",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DocumentRefreshResponse,
                    construct_type(
                        type_=DocumentRefreshResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def compute_rag_index(
        self,
        documentation_id: str,
        *,
        model: EmbeddingModelEnum,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[RagDocumentIndexResponseModel]:
        """
        In case the document is not RAG indexed, it triggers rag indexing task, otherwise it just returns the current status.

        Parameters
        ----------
        documentation_id : str
            The id of a document from the knowledge base. This is returned on document addition.

        model : EmbeddingModelEnum

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RagDocumentIndexResponseModel]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v1/convai/knowledge-base/{jsonable_encoder(documentation_id)}/rag-index",
            method="POST",
            json={
                "model": model,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RagDocumentIndexResponseModel,
                    construct_type(
                        type_=RagDocumentIndexResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawDocumentClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def update_file(
        self, documentation_id: str, *, file: core.File, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DocumentUpdateFileResponse]:
        """
        Update the source file of a file document. The document name, content, and metadata are updated to reflect the new file. Any manual content edits will be overwritten.

        Parameters
        ----------
        documentation_id : str
            The id of a document from the knowledge base. This is returned on document addition.

        file : core.File
            See core.File for more documentation

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DocumentUpdateFileResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/convai/knowledge-base/{jsonable_encoder(documentation_id)}/update-file",
            method="PATCH",
            data={},
            files={
                "file": file,
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DocumentUpdateFileResponse,
                    construct_type(
                        type_=DocumentUpdateFileResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def refresh(
        self, documentation_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DocumentRefreshResponse]:
        """
        Manually refresh a URL document by re-fetching its content from the source URL.

        Parameters
        ----------
        documentation_id : str
            The id of a document from the knowledge base. This is returned on document addition.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DocumentRefreshResponse]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/convai/knowledge-base/{jsonable_encoder(documentation_id)}/refresh",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DocumentRefreshResponse,
                    construct_type(
                        type_=DocumentRefreshResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def compute_rag_index(
        self,
        documentation_id: str,
        *,
        model: EmbeddingModelEnum,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[RagDocumentIndexResponseModel]:
        """
        In case the document is not RAG indexed, it triggers rag indexing task, otherwise it just returns the current status.

        Parameters
        ----------
        documentation_id : str
            The id of a document from the knowledge base. This is returned on document addition.

        model : EmbeddingModelEnum

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RagDocumentIndexResponseModel]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v1/convai/knowledge-base/{jsonable_encoder(documentation_id)}/rag-index",
            method="POST",
            json={
                "model": model,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RagDocumentIndexResponseModel,
                    construct_type(
                        type_=RagDocumentIndexResponseModel,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        construct_type(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        KnowledgeBaseGetOrCreateRagIndexesResponseValue,
        KnowledgeBaseGetOrCreateRagIndexesResponseValue_Failure,
        KnowledgeBaseGetOrCreateRagIndexesResponseValue_Success,
    )
    from . import document, documents
    from .document import (
        DocumentRefreshResponse,
        DocumentRefreshResponse_File,
        DocumentRefreshResponse_Folder,
        DocumentRefreshResponse_Text,
        DocumentRefreshResponse_Url,
        DocumentUpdateFileResponse,
        DocumentUpdateFileResponse_File,
        DocumentUpdateFileResponse_Folder,
        DocumentUpdateFileResponse_Text,
        DocumentUpdateFileResponse_Url,
    )
    from .documents import (
        DocumentsGetResponse,
        DocumentsGetResponse_File,
        DocumentsGetResponse_Folder,
        DocumentsGetResponse_Text,
        DocumentsGetResponse_Url,
        DocumentsUpdateResponse,
        DocumentsUpdateResponse_File,
        DocumentsUpdateResponse_Folder,
        DocumentsUpdateResponse_Text,
        DocumentsUpdateResponse_Url,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "DocumentRefreshResponse": ".document",
    "DocumentRefreshResponse_File": ".document",
    "DocumentRefreshResponse_Folder": ".document",
    "DocumentRefreshResponse_Text": ".document",
    "DocumentRefreshResponse_Url": ".document",
    "DocumentUpdateFileResponse": ".document",
    "DocumentUpdateFileResponse_File": ".document",
    "DocumentUpdateFileResponse_Folder": ".document",
    "DocumentUpdateFileResponse_Text": ".document",
    "DocumentUpdateFileResponse_Url": ".document",
    "DocumentsGetResponse": ".documents",
    "DocumentsGetResponse_File": ".documents",
    "DocumentsGetResponse_Folder": ".documents",
    "DocumentsGetResponse_Text": ".documents",
    "DocumentsGetResponse_Url": ".documents",
    "DocumentsUpdateResponse": ".documents",
    "DocumentsUpdateResponse_File": ".documents",
    "DocumentsUpdateResponse_Folder": ".documents",
    "DocumentsUpdateResponse_Text": ".documents",
    "DocumentsUpdateResponse_Url": ".documents",
    "KnowledgeBaseGetOrCreateRagIndexesResponseValue": ".types",
    "KnowledgeBaseGetOrCreateRagIndexesResponseValue_Failure": ".types",
    "KnowledgeBaseGetOrCreateRagIndexesResponseValue_Success": ".types",
    "document": ".document",
    "documents": ".documents",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "DocumentRefreshResponse",
    "DocumentRefreshResponse_File",
    "DocumentRefreshResponse_Folder",
    "DocumentRefreshResponse_Text",
    "DocumentRefreshResponse_Url",
    "DocumentUpdateFileResponse",
    "DocumentUpdateFileResponse_File",
    "DocumentUpdateFileResponse_Folder",
    "DocumentUpdateFileResponse_Text",
    "DocumentUpdateFileResponse_Url",
    "DocumentsGetResponse",
    "DocumentsGetResponse_File",
    "DocumentsGetResponse_Folder",
    "DocumentsGetResponse_Text",
    "DocumentsGetResponse_Url",
    "DocumentsUpdateResponse",
    "DocumentsUpdateResponse_File",
    "DocumentsUpdateResponse_Folder",
    "DocumentsUpdateResponse_Text",
    "DocumentsUpdateResponse_Url",
    "KnowledgeBaseGetOrCreateRagIndexesResponseValue",
    "KnowledgeBaseGetOrCreateRagIndexesResponseValue_Failure",
    "KnowledgeBaseGetOrCreateRagIndexesResponseValue_Success",
    "document",
    "documents",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .dependent_phone_number_identifier import DependentPhoneNumberIdentifier
from .get_secret_dependencies_response_model_dependencies_one_item import (
    GetSecretDependenciesResponseModelDependenciesOneItem,
)
from .get_secret_dependencies_response_model_dependencies_zero_item import (
    GetSecretDependenciesResponseModelDependenciesZeroItem,
)

GetSecretDependenciesResponseModelDependencies = typing.Union[
    typing.List[GetSecretDependenciesResponseModelDependenciesZeroItem],
    typing.List[GetSecretDependenciesResponseModelDependenciesOneItem],
    typing.List[DependentPhoneNumberIdentifier],
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel


class SkipTurnToolConfig(UncheckedBaseModel):
    """
    Allows the agent to explicitly skip its turn.

    This tool should be invoked by the LLM when the user indicates they would like
    to think or take a short pause before continuing the conversation—e.g. when
    they say: "Give me a second", "Let me think", or "One moment please".  After
    calling this tool, the assistant should not speak until the user speaks
    again, or another normal turn-taking condition is met.  The tool itself has
    no parameters and performs no side-effects other than informing the backend
    that the current turn generation is complete.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, update_forward_refs
from ..core.unchecked_base_model import UncheckedBaseModel
from .agent_failure_response_example import AgentFailureResponseExample
from .agent_successful_response_example import AgentSuccessfulResponseExample
from .conversation_history_transcript_common_model_output import ConversationHistoryTranscriptCommonModelOutput
from .conversation_initiation_source import ConversationInitiationSource
from .dynamic_variable_value_type_output import DynamicVariableValueTypeOutput
from .test_from_conversation_metadata_output import TestFromConversationMetadataOutput


class GetResponseUnitTestResponseModel(UncheckedBaseModel):
    from_conversation_metadata: typing.Optional[TestFromConversationMetadataOutput] = pydantic.Field(default=None)
    """
    Metadata of a conversation this test was created from (if applicable).
    """

    dynamic_variables: typing.Optional[typing.Dict[str, typing.Optional[DynamicVariableValueTypeOutput]]] = (
        pydantic.Field(default=None)
    )
    """
    Dynamic variables to replace in the agent config during testing
    """

    chat_history: typing.Optional[typing.List[ConversationHistoryTranscriptCommonModelOutput]] = None
    conversation_initiation_source: typing.Optional[ConversationInitiationSource] = pydantic.Field(default=None)
    """
    Simulate the test as if the conversation originated from this channel.
    """

    success_condition: typing.Optional[str] = pydantic.Field(default=None)
    """
    A prompt that evaluates whether the agent's response is successful. Should return True or False.
    """

    success_examples: typing.Optional[typing.List[AgentSuccessfulResponseExample]] = pydantic.Field(default=None)
    """
    Non-empty list of example responses that should be considered successful
    """

    failure_examples: typing.Optional[typing.List[AgentFailureResponseExample]] = pydantic.Field(default=None)
    """
    Non-empty list of example responses that should be considered failures
    """

    id: str
    name: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


update_forward_refs(GetResponseUnitTestResponseModel)



# This file was auto-generated by Fern from our API Definition.

import typing

ConversationSummaryResponseModelStatus = typing.Union[
    typing.Literal["initiated", "in-progress", "processing", "done", "failed"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .contributor import Contributor
from .direct_publishing_read_response_model_display_mode import DirectPublishingReadResponseModelDisplayMode
from .direct_publishing_read_response_model_genre_item import DirectPublishingReadResponseModelGenreItem
from .direct_publishing_read_response_model_payout_type import DirectPublishingReadResponseModelPayoutType
from .direct_publishing_read_response_model_target_audience import DirectPublishingReadResponseModelTargetAudience
from .preview_audio_db_model import PreviewAudioDbModel
from .read_legal_terms import ReadLegalTerms
from .read_metadata_chapter_db_model import ReadMetadataChapterDbModel
from .review_response_model import ReviewResponseModel
from .sample_config_db_model import SampleConfigDbModel


class DirectPublishingReadResponseModel(UncheckedBaseModel):
    read_id: str
    created_at_unix: int
    updated_at_unix: int
    word_count: int
    char_count: int
    chapters: typing.List[ReadMetadataChapterDbModel]
    title: typing.Optional[str] = None
    author: typing.Optional[str] = None
    description: typing.Optional[str] = None
    article_image_url: typing.Optional[str] = None
    language: typing.Optional[str] = None
    locale: typing.Optional[str] = None
    display_mode: typing.Optional[DirectPublishingReadResponseModelDisplayMode] = None
    genre: typing.Optional[typing.List[DirectPublishingReadResponseModelGenreItem]] = None
    fiction: typing.Optional[str] = None
    content_type: typing.Optional[str] = None
    original_file_type: typing.Optional[str] = None
    target_audience: typing.Optional[DirectPublishingReadResponseModelTargetAudience] = None
    mature_content: typing.Optional[bool] = None
    safesearch_adult: typing.Optional[bool] = None
    origin: typing.Optional[str] = None
    publication_date: typing.Optional[str] = None
    isbn: typing.Optional[str] = None
    ean: typing.Optional[str] = None
    legal_terms: typing.Optional[ReadLegalTerms] = None
    content_guidelines_terms: typing.Optional[ReadLegalTerms] = None
    last_updated_from_project_unix: typing.Optional[int] = None
    publishing_project_id: typing.Optional[str] = None
    publishing_state: typing.Optional[str] = None
    publisher_profile_id: typing.Optional[str] = None
    quality_score: typing.Optional[int] = None
    publisher: typing.Optional[str] = None
    copyright: typing.Optional[str] = None
    subtitle: typing.Optional[str] = None
    distribution_territories: typing.Optional[typing.List[str]] = None
    edition: typing.Optional[str] = None
    contributors: typing.Optional[typing.List[Contributor]] = None
    payout_type: typing.Optional[DirectPublishingReadResponseModelPayoutType] = None
    list_price: typing.Optional[float] = None
    currency: typing.Optional[typing.Literal["usd"]] = None
    original_audio_project_export_id: typing.Optional[str] = None
    original_audio_document_id: typing.Optional[str] = None
    series_id: typing.Optional[str] = None
    volume: typing.Optional[int] = None
    published_at_unix: typing.Optional[int] = None
    read_slug: typing.Optional[str] = None
    preview_audio_object: typing.Optional[PreviewAudioDbModel] = None
    sample_config: typing.Optional[SampleConfigDbModel] = None
    review: typing.Optional[ReviewResponseModel] = None
    voice_id: typing.Optional[str] = None
    can_use_assistant: typing.Optional[bool] = None
    is_voice_changer_on: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .api_integration_o_auth_2_custom_app_response_scope_separator import (
    ApiIntegrationOAuth2CustomAppResponseScopeSeparator,
)
from .auth_connection_dependencies import AuthConnectionDependencies
from .o_auth_connection_status import OAuthConnectionStatus


class ApiIntegrationOAuth2CustomAppResponse(UncheckedBaseModel):
    """
    Response model for user-owned OAuth2 Custom App auth connections
    """

    name: str
    provider: str
    token_url: str
    scopes: typing.Optional[typing.List[str]] = None
    scope_separator: typing.Optional[ApiIntegrationOAuth2CustomAppResponseScopeSeparator] = pydantic.Field(default=None)
    """
    Separator for scopes
    """

    expires_at: str = pydantic.Field()
    """
    ISO 8601 timestamp of when the access token expires
    """

    integration_id: str
    credential_id: str
    status: typing.Optional[OAuthConnectionStatus] = pydantic.Field(default=None)
    """
    Current health status of the OAuth connection
    """

    status_detail: typing.Optional[str] = pydantic.Field(default=None)
    """
    Human-readable detail about the current status, e.g. the error message on refresh failure
    """

    status_updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    ISO 8601 timestamp of the last status change
    """

    client_id: str = pydantic.Field()
    """
    OAuth client ID (rendered from template if credential uses templated credentials, None for legacy connections)
    """

    id: str
    used_by: typing.Optional[AuthConnectionDependencies] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.unchecked_base_model import UncheckedBaseModel
from .video_key_moment import VideoKeyMoment
from .video_segment import VideoSegment
from .video_subject import VideoSubject


class VideoAnalysisResult(UncheckedBaseModel):
    title: str
    description: str
    content_type: typing.Optional[str] = None
    overall_pacing: typing.Optional[str] = None
    subjects: typing.Optional[typing.List[VideoSubject]] = None
    segments: typing.Optional[typing.List[VideoSegment]] = None
    key_moments: typing.Optional[typing.List[VideoKeyMoment]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
